package com.incubator.game.data.mq;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;

public class RabbitMQUtil {

    private static final String EXCHANGE_NAME = "zj_exchange";
    private static final String QUEUE_NAME = "zj_queue";
    private static final String ROUTING_KEY = "zj_routing_key";

    private static Connection connection;
    private static Channel channel;

    static {
        try {
            // 初始化连接和通道
            initializeConnectionAndChannel();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void initializeConnectionAndChannel() throws Exception {
        ConnectionFactory factory = new ConnectionFactory();
        factory.setHost("localhost");
        factory.setPort(5672);
        factory.setUsername("guest");
        factory.setPassword("guest");

        connection = factory.newConnection();
        channel = connection.createChannel();

        // 声明交换机
        channel.exchangeDeclare(EXCHANGE_NAME, "direct",true);

        // 声明队列
        channel.queueDeclare(QUEUE_NAME, true, false, false, null);

        // 绑定队列到交换机
        channel.queueBind(QUEUE_NAME, EXCHANGE_NAME, ROUTING_KEY);
    }

    public static void sendMessage(String message) throws Exception {
        // 发送消息
        channel.basicPublish(EXCHANGE_NAME, ROUTING_KEY, null, message.getBytes());
        System.out.println("发送消息: " + message);
    }

    public static void closeConnection() {
        try {
            if (channel != null && channel.isOpen()) {
                channel.close();
            }
            if (connection != null && connection.isOpen()) {
                connection.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}