package com.incubator.game.util;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 掼蛋 工具类
 */
public final class GDUtils {

    /**
     * 掼蛋对应的数值
     * 0x02-0x0e 方片（♦️） 2(2) 3(3) 4(4) 5(5) 6(6) 7(7) 8(8) 9(9) 10(10) J(11) Q(12) K(13) A(14)
     * 0x12-0x1e 梅花（♣️） 2(18) 3(19) 4(20) 5(21) 6(22) 7(23) 8(24) 9(25) 10(26) J(27) Q(28) K(29) A(30)
     * 0x22-0x2e 红桃（♥️） 2(34) 3(35) 4(36) 5(37) 6(38) 7(39) 8(40) 9(41) 10(42) J(43) Q(44) K(45) A(46)
     * 0x32-0x3e 黑桃（♠️） 2(50) 3(51) 4(52) 5(53) 6(54) 7(55) 8(56) 9(57) 10(58) J(59) Q(60) K(61) A(62)
     * 0xef      小王(239)
     * 0xff      大王(255)
     */
    public static Integer[] values = {
            0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e,
            0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e,
            0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e,
            0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e,
            0xef,
            0xff
    };

    /** 大王数值 **/
    public static int BIGKING = 0xff;

    /** 随机队友-系统翻牌 2、3、4、5、6、7、8、9、10、J、Q、K、A **/
    public static Integer[] randomTeam = {
            0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e
    };

    /** 级数配置 2、3、4、5、6、7、8、9、10、J、Q、K、A **/
    public static int[] levelPoint = {
            2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14
    };

    // 牌型枚举
    public enum CardType {
        SINGLE,              // 单张
        PAIR,                // 对子
        PAIR_SEQUENCE,       // 3连对(3个连续对子)
        TRIPLE,              // 三不带
        TRIPLE_WITH_TWO,     // 三带二
        TRIPLE_SEQUENCE,     // 三顺(二个连续三张)
        STRAIGHT,            // 顺子(5张牌)
        BOMB,                // 通用炸弹
        FOUR_BOMB,           // 四炸弹
        FIVE_BOMB,           // 五炸弹
        SIX_BOMB,            // 六炸弹
        SEVEN_BOMB,          // 七炸弹
        EIGHT_BOMB,          // 八炸弹
        FOUR_KINGS,          // 四王炸
        ROYAL_FLUSH,         // 同花顺（可选）
        INVALID              // 无效牌型
    }

    /**
     * 是否是炸弹
     *
     * @param type
     * @return
     */
    public static boolean isBomb(CardType type) {
        return type == CardType.BOMB
                || type == CardType.FOUR_BOMB
                || type == CardType.FIVE_BOMB
                || type == CardType.SIX_BOMB
                || type == CardType.SEVEN_BOMB
                || type == CardType.EIGHT_BOMB
                || type == CardType.FOUR_KINGS
                || type == CardType.ROYAL_FLUSH;
    }

    /**
     * 炸弹是否计算倍数
     *
     * @param type
     * @return
     */
    public static boolean isBombCalculateMultiples(CardType type) {
        return type == CardType.SIX_BOMB
                || type == CardType.SEVEN_BOMB
                || type == CardType.EIGHT_BOMB
                || type == CardType.FOUR_KINGS;
    }

    /**
     * 进还贡时，获取手牌最大的牌（过滤红星级牌）
     *
     * @param cards 玩家手牌
     * @param curLevelPoint 当前级牌点数
     * @return
     */
    public static int getTributeMaxCard(int[] cards, int curLevelPoint) {
        int maxCards = 0;
        for (Integer card : cards) {
            byte low = (byte) getCardPoint(card, curLevelPoint);
            byte high = (byte) (card >> 4);
            // 过滤红星级牌
            if (GDUtils.isWildCard(card, curLevelPoint)) {
                continue;
            }
            if (low > (byte) (maxCards & 0xf)) {
                maxCards = card;
            } else if (low == (byte) (maxCards & 0xf)) {
                if (high > (byte) (maxCards >> 4)) {
                    maxCards = card;
                }
            }
        }
        return maxCards;
    }

    /**
     * 随机队友-亮牌
     *
     * @return
     */
    public static int randomTeamShowCards() {
        List<Integer> randomTeam = new ArrayList<>(Arrays.asList(GDUtils.randomTeam));
        Collections.shuffle(randomTeam);
        return randomTeam.get(0);
    }

    /**
     * 是否为赖子 红星级牌
     *
     * @param card
     * @param curLevelPoint
     * @return
     */
    public static boolean isWildCard(int card, int curLevelPoint) {
        byte low = (byte) (card & 0xF);
        byte high = (byte) (card >> 4);
        // 红心级牌
        return high == (byte) 2 && low == (byte) curLevelPoint;
    }

    /**
     * 判断玩家出的牌是否可以压过上一手牌
     *
     * @param previousCards 上一手牌
     * @param previousCardsType 上一手牌的牌型
     * @param disCardsList 当前玩家出的牌型
     * @param disCardsType 当前玩家出的牌型
     * @param currentLevel 当前级数
     * @param wildCard 赖子牌
     * @return 是否可以出牌
     */
    public static boolean canDisCard(int[] previousCards, CardType previousCardsType,
                                     int[] disCardsList, GDUtils.CardType disCardsType,
                                     int currentLevel,
                                     int wildCard) {
        // 0.桌上无牌直接出
        if (previousCards == null || previousCardsType == null) {
            return true;
        }

        // 1.牌型不一样,判断手牌是否是炸弹
        if (disCardsType != previousCardsType && !GDUtils.isBomb(disCardsType)) {
            return false;
        }

        // 获取权重值
        int previousWeight = GDUtils.getCardTypeWeight(previousCardsType);
        int currentWeight = GDUtils.getCardTypeWeight(disCardsType);

        // 权重不同，按权重大小比较
        if (currentWeight > previousWeight) {
            return true;
        }

        // 权重相同，根据牌点大小进行比较（仅适用于同类型的牌型，如炸弹或顺子）
        if (currentWeight == previousWeight) {
            return GDUtils.compareCardStrength(previousCards, disCardsList, currentLevel, disCardsType, wildCard) > 0;
        }

        return false; // 其他情况不能出牌
    }

    /**
     * 比较两组牌的大小（仅在牌型相同时调用）
     *
     * @param previousCards 上一手牌
     * @param disCardList 当前玩家出的牌
     * @param curLevel 当前级数
     * @param disCardsType 出牌牌型
     * @param wildCard 赖子牌
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    public static int compareCardStrength(int[] previousCards, int[] disCardList, int curLevel, CardType disCardsType, int wildCard) {
        List<Integer> previousPoints = Arrays.stream(previousCards)
                .map(card -> GDUtils.getCardPointWildCard(card, wildCard, curLevel))
                .sorted()
                .boxed()
                .collect(Collectors.toList());

        List<Integer> currentPoints = Arrays.stream(disCardList)
                .map(card -> GDUtils.getCardPointWildCard(card, wildCard, curLevel))
                .sorted()
                .boxed()
                .collect(Collectors.toList());

        int wildCardCountCurrent = countWildCards(currentPoints, -1);
        int wildCardCountPrevious = countWildCards(previousPoints, -1);

        switch (disCardsType) {
            case SINGLE:
                // 单张直接比较最大点数
                return compareSingle(previousPoints, currentPoints, curLevel);
            case PAIR:
                // 对子：只比较两张部分
                return comparePair(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case PAIR_SEQUENCE:
                // 3连对
                return comparePairSequence(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case TRIPLE:
                // 三不带：考虑赖子的补充逻辑
                return compareTriple(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case TRIPLE_WITH_TWO:
                // 三带二：三张部分优先考虑赖子牌的补充
                return compareTripleWithTwo(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case TRIPLE_SEQUENCE:
                // 三顺(二个连续三张)
                return compareThreeConsecutive(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case STRAIGHT:
                // 顺子：赖子牌可以补齐顺子
                return compareStraight(previousPoints, currentPoints, curLevel, wildCardCountCurrent, wildCardCountPrevious);
            case BOMB:
                // 炸弹：赖子牌可以增强炸弹的级别
                return compareBomb(previousPoints, currentPoints, wildCardCountCurrent, wildCardCountPrevious);
            default:
                // 默认按最大点数比较
                return Integer.compare(
                        currentPoints.get(currentPoints.size() - 1),
                        previousPoints.get(previousPoints.size() - 1)
                );
        }
    }

    /**
     * 统计赖子牌数量
     */
    private static int countWildCards(List<Integer> points, int wildCard) {
        return (int) points.stream().filter(card -> card == wildCard).count();
    }

    /**
     * 替换赖子牌为当前手牌中的任意其他牌（排除大王和小王）
     *
     * @param points 当前手牌的点数列表
     * @param wildCardCount 当前赖子牌的数量
     * @param curLevel 当前级数（主牌）
     * @param requiredCount 需要的牌型数量（例如三张牌需要 3）
     * @return 调整后的点数列表
     */
    private static List<Integer> replaceWildCardsWithTrump(List<Integer> points, int wildCardCount, int curLevel, int requiredCount) {
        List<Integer> adjustedPoints = new ArrayList<>(points);
        adjustedPoints.removeIf(p -> p == -1); // 移除赖子标识

        // 找到当前手牌中排除大王、小王的最小点数
        List<Integer> availablePoints = adjustedPoints.stream()
                .filter(p -> p != 16 && p != 17)  // 排除大王（17）和小王（16）
                .distinct()  // 确保不重复
                .sorted()
                .collect(Collectors.toList());

        // 如果有可用的牌点，则选择最小的点数作为赖子的替换点
        int replacementPoint = availablePoints.isEmpty() ? curLevel : availablePoints.get(0); // 默认选择最小点数，如果没有则选择当前级牌

        // 替换赖子牌为选定的点数
        while (wildCardCount > 0 && adjustedPoints.size() < requiredCount) {
            adjustedPoints.add(replacementPoint); // 使用替换点数填充赖子
            wildCardCount--;
        }

        adjustedPoints.sort(Collections.reverseOrder()); // 按照点数降序排序
        return adjustedPoints;
    }

    /**
     * 比较单张牌的大小
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareSingle(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel) {
        // 大王和小王的特殊点数
        final int BIG_JOKER = 17; // 大王
        final int SMALL_JOKER = 16; // 小王

        // 获取上一手牌和当前手牌的最大点数
        int maxPrevious = previousPoints.stream()
                .mapToInt(p -> p == -1 ? curLevel : p) // 将赖子牌视为主牌点数
                .max()
                .orElse(-1);

        int maxCurrent = currentPoints.stream()
                .mapToInt(p -> p == -1 ? curLevel : p) // 将赖子牌视为主牌点数
                .max()
                .orElse(-1);

        // 判断是否有大王或小王
        boolean previousHasBigJoker = previousPoints.contains(BIG_JOKER);
        boolean previousHasSmallJoker = previousPoints.contains(SMALL_JOKER);
        boolean currentHasBigJoker = currentPoints.contains(BIG_JOKER);
        boolean currentHasSmallJoker = currentPoints.contains(SMALL_JOKER);

        // 优先级比较：大王 > 小王 > 主牌
        if (currentHasBigJoker && !previousHasBigJoker) {
            return 1; // 当前牌含大王，优先级更高
        }
        if (!currentHasBigJoker && previousHasBigJoker) {
            return -1; // 上一手牌含大王，优先级更高
        }
        if (currentHasSmallJoker && !previousHasSmallJoker) {
            return 1; // 当前牌含小王，优先级更高
        }
        if (!currentHasSmallJoker && previousHasSmallJoker) {
            return -1; // 上一手牌含小王，优先级更高
        }

        // 主牌优先级比较
        boolean previousIsTrump = (maxPrevious == curLevel);
        boolean currentIsTrump = (maxCurrent == curLevel);

        if (currentIsTrump && !previousIsTrump) {
            return 1; // 当前牌是主牌，优先级更高
        }
        if (!currentIsTrump && previousIsTrump) {
            return -1; // 上一手牌是主牌，优先级更高
        }

        // 默认比较点数
        return Integer.compare(maxCurrent, maxPrevious);
    }

    /**
     * 比较三连对牌型的大小（考虑赖子牌）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int comparePairSequence(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌点数，形成有效的对子序列
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 2);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 2);

        // 判断每个牌型的最大点数
        int maxPrevious = getPairValue(adjustedPrevious);
        int maxCurrent = getPairValue(adjustedCurrent);

        // 主牌优先级比较
        boolean previousIsTrump = (maxPrevious == curLevel);
        boolean currentIsTrump = (maxCurrent == curLevel);

        if (currentIsTrump && !previousIsTrump) {
            return 1; // 当前牌是主牌对子，优先级更高
        }
        if (!currentIsTrump && previousIsTrump) {
            return -1; // 上一手牌是主牌对子，优先级更高
        }

        // 默认比较对子序列的最大点数（最后一个对的点数）
        return Integer.compare(maxCurrent, maxPrevious);
    }

    /**
     * 比较对子牌的大小
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int comparePair(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 特殊点数处理：大王和小王
        final int BIG_JOKER = 17; // 大王
        final int SMALL_JOKER = 16; // 小王

        // 检查上一手牌和当前手牌是否包含大王或小王
        boolean previousHasBigJoker = previousPoints.contains(BIG_JOKER);
        boolean previousHasSmallJoker = previousPoints.contains(SMALL_JOKER);
        boolean currentHasBigJoker = currentPoints.contains(BIG_JOKER);
        boolean currentHasSmallJoker = currentPoints.contains(SMALL_JOKER);

        // 优先比较大王和小王
        if (currentHasBigJoker && !previousHasBigJoker) return 1;
        if (!currentHasBigJoker && previousHasBigJoker) return -1;
        if (currentHasSmallJoker && !previousHasSmallJoker) return 1;
        if (!currentHasSmallJoker && previousHasSmallJoker) return -1;

        // 补充赖子牌以形成对子
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 2);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 2);

        // 检查并获取每手牌的最大对子点数
        int maxPreviousPair = getPairValue(adjustedPrevious);
        int maxCurrentPair = getPairValue(adjustedCurrent);

        // 主牌优先级比较（如果最大对子点数等于当前级数，则认为它是主牌对子）
        boolean previousIsTrumpPair = (maxPreviousPair == curLevel);
        boolean currentIsTrumpPair = (maxCurrentPair == curLevel);

        if (currentIsTrumpPair && !previousIsTrumpPair) {
            return 1; // 当前牌是主牌对子，优先级更高
        }
        if (!currentIsTrumpPair && previousIsTrumpPair) {
            return -1; // 上一手牌是主牌对子，优先级更高
        }

        // 默认比较对子点数
        return Integer.compare(maxCurrentPair, maxPreviousPair);
    }

    /**
     * 获取对子的最大点数（假设列表已经是对子形式或调整后的形式）
     *
     * @param points 调整后的点数列表
     * @return 对子的最大点数
     */
    private static int getPairValue(List<Integer> points) {
        // 假设points已经通过replaceWildCardsWithTrump调整为至少有两个元素
        // 并且已经排序，我们直接返回第二个元素（即较大的那个）
        // 因为对子中两个点数应该相同，返回任意一个即可（这里选择较大的，即第二个）
        // 如果不是严格对子（比如通过赖子补充），返回最大点数作为代表
        if (points.size() >= 2) {
            // 如果存在多个相同的最大点数，返回其中一个即可
            // 这里简单返回最大的那个，因为已经排序
            return points.get(points.size() - 1);
        }
        // 理论上不应该发生，因为replaceWildCardsWithTrump应保证至少有两个元素
        throw new IllegalArgumentException("点数列表应至少包含两个元素以形成对子");
    }

    /**
     * 比较三不带牌型的大小
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareTriple(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌点数，形成有效的三张牌
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 3);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 3);

        // 获取每手牌的最大三张点数
        int maxPreviousTriple = getMaxFrequencyCard(adjustedPrevious, 3);
        int maxCurrentTriple = getMaxFrequencyCard(adjustedCurrent, 3);

        // 主牌优先级比较
        boolean previousIsTrumpTriple = (maxPreviousTriple == curLevel);
        boolean currentIsTrumpTriple = (maxCurrentTriple == curLevel);

        if (currentIsTrumpTriple && !previousIsTrumpTriple) {
            return 1; // 当前牌是主牌三张，优先级更高
        }
        if (!currentIsTrumpTriple && previousIsTrumpTriple) {
            return -1; // 上一手牌是主牌三张，优先级更高
        }

        // 默认比较三张点数
        return Integer.compare(maxCurrentTriple, maxPreviousTriple);
    }

    /**
     * 比较三带二牌型的大小（仅比较三张牌的部分，忽略两张牌）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareTripleWithTwo(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌点数，形成有效的三带二牌型（只关注三张牌的部分）
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 3);  // 只需三张
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 3);  // 只需三张

        // 获取三张牌的最大点数
        int maxPreviousTriple = getMaxFrequencyCard(adjustedPrevious, 3);
        int maxCurrentTriple = getMaxFrequencyCard(adjustedCurrent, 3);

        // 主牌优先级比较
        boolean previousIsTrumpTriple = (maxPreviousTriple == curLevel);
        boolean currentIsTrumpTriple = (maxCurrentTriple == curLevel);

        if (currentIsTrumpTriple && !previousIsTrumpTriple) {
            return 1; // 当前牌是主牌三张，优先级更高
        }
        if (!currentIsTrumpTriple && previousIsTrumpTriple) {
            return -1; // 上一手牌是主牌三张，优先级更高
        }

        // 默认比较三张点数（忽略两张部分）
        return Integer.compare(maxCurrentTriple, maxPreviousTriple);
    }

    /**
     * 比较两组三张连续牌型的大小，支持赖子牌的处理
     *
     * @param previousPoints 上一手牌的点数列表（三张连续牌）
     * @param currentPoints 当前手牌的点数列表（三张连续牌）
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareThreeConsecutive(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌点数，形成有效的三张连续牌
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 3);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 3);

        // 判断上一手牌和当前手牌的强度
        int previousStrength = calculateHandStrength(adjustedPrevious, wildCardCountPrevious);
        int currentStrength = calculateHandStrength(adjustedCurrent, wildCardCountCurrent);

        // 如果当前牌比上一手牌强，返回1
        if (currentStrength > previousStrength) {
            return 1;
        }
        // 如果上一手牌比当前牌强，返回-1
        if (currentStrength < previousStrength) {
            return -1;
        }

        // 如果两手牌的强度相同，按点数进行比较
        return compareByPoints(adjustedPrevious, adjustedCurrent);
    }

    /**
     * 计算手牌的强度（考虑赖子牌替换后的牌型）
     * @param points 调整后的点数列表
     * @param wildCardCount 赖子牌数量
     * @return 该手牌的强度值
     */
    private static int calculateHandStrength(List<Integer> points, int wildCardCount) {
        // 如果有赖子牌，根据赖子的数量来决定牌型的强度
        if (wildCardCount == 0) {
            // 没有赖子牌，则返回手牌的最大点数
            return points.get(points.size() - 1);
        }

        // 赖子牌数量 > 0 时，尝试使用赖子提升最大点数
        int adjustedStrength = points.get(points.size() - 1); // 先按最大牌计算
        // 如果赖子数量比较多，可能有机会改变牌型的强度，考虑赖子的替换效果
        if (wildCardCount > 1) {
            adjustedStrength += 1;  // 假设通过赖子牌，我们可以增加一张更高的牌
        }
        return adjustedStrength;
    }

    /**
     * 比较两手牌的点数
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareByPoints(List<Integer> previousPoints, List<Integer> currentPoints) {
        // 先比较最大点数
        int maxPrevious = getMaxConsecutiveValue(previousPoints);
        int maxCurrent = getMaxConsecutiveValue(currentPoints);

        // 如果当前牌的最大点数较大，返回1
        if (maxCurrent > maxPrevious) {
            return 1;
        }
        // 如果上一手牌的最大点数较大，返回-1
        if (maxCurrent < maxPrevious) {
            return -1;
        }

        // 如果最大点数相同，比较最小点数
        int minPrevious = getMinConsecutiveValue(previousPoints);
        int minCurrent = getMinConsecutiveValue(currentPoints);

        return Integer.compare(minCurrent, minPrevious);
    }

    /**
     * 获取连续三张牌的最大点数（即三张中的最大牌）
     *
     * @param points 调整后的点数列表
     * @return 三张牌中的最大点数
     */
    private static int getMaxConsecutiveValue(List<Integer> points) {
        return points.get(points.size() - 1); // 假设牌已经是顺序排列
    }

    /**
     * 获取连续三张牌的最小点数（即三张中的最小牌）
     *
     * @param points 调整后的点数列表
     * @return 三张牌中的最小点数
     */
    private static int getMinConsecutiveValue(List<Integer> points) {
        return points.get(0); // 假设牌已经是顺序排列
    }

    /**
     * 比较顺子牌型的大小（考虑赖子牌的补充）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param curLevel 当前级数（主牌）
     * @param wildCardCountCurrent 当前手牌的赖子数量
     * @param wildCardCountPrevious 上一手牌的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareStraight(List<Integer> previousPoints, List<Integer> currentPoints, int curLevel, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 替换赖子牌为当前级牌点数，形成有效的顺子
        List<Integer> adjustedPrevious = replaceWildCardsWithTrump(previousPoints, wildCardCountPrevious, curLevel, 5);
        List<Integer> adjustedCurrent = replaceWildCardsWithTrump(currentPoints, wildCardCountCurrent, curLevel, 5);

        // 获取每手牌的最大顺子点数
        int maxPreviousStraight = getMaxStraightValue(adjustedPrevious);
        int maxCurrentStraight = getMaxStraightValue(adjustedCurrent);

        // 如果当前顺子的最大点数较大，返回1
        if (maxCurrentStraight > maxPreviousStraight) {
            return 1;
        }
        // 如果上一手牌的最大点数较大，返回-1
        if (maxCurrentStraight < maxPreviousStraight) {
            return -1;
        }

        // 如果顺子的最大点数相同，比较顺子的最小点数（头牌）
        int minPreviousStraight = getMinStraightValue(adjustedPrevious);
        int minCurrentStraight = getMinStraightValue(adjustedCurrent);

        return Integer.compare(minCurrentStraight, minPreviousStraight);
    }

    /**
     * 获取顺子的最大点数（即顺子中的最大牌）
     *
     * @param points 调整后的点数列表
     * @return 顺子的最大点数
     */
    private static int getMaxStraightValue(List<Integer> points) {
        // 顺子应该由连续的五张牌组成，获取顺子的最大点数
        return points.get(points.size() - 1);
    }

    /**
     * 获取顺子的最小点数（即顺子中的最小牌）
     *
     * @param points 调整后的点数列表
     * @return 顺子的最小点数
     */
    private static int getMinStraightValue(List<Integer> points) {
        // 顺子应该由连续的五张牌组成，获取顺子的最小点数
        return points.get(0);
    }

    /**
     * 比较炸弹牌型的大小（考虑赖子牌的增强）
     *
     * @param previousPoints 上一手牌的点数列表
     * @param currentPoints 当前手牌的点数列表
     * @param wildCardCountCurrent 当前手牌中的赖子数量
     * @param wildCardCountPrevious 上一手牌中的赖子数量
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    private static int compareBomb(List<Integer> previousPoints, List<Integer> currentPoints, int wildCardCountCurrent, int wildCardCountPrevious) {
        // 获取上一手牌和当前手牌中的最大炸弹点数
        int maxPreviousBomb = getMaxFrequencyCard(previousPoints, 4);
        int maxCurrentBomb = getMaxFrequencyCard(currentPoints, 4);

        // 如果当前手牌的炸弹点数较大，返回1
        if (maxCurrentBomb > maxPreviousBomb) {
            return 1;
        }
        // 如果上一手牌的炸弹点数较大，返回-1
        if (maxCurrentBomb < maxPreviousBomb) {
            return -1;
        }

        // 如果炸弹点数相同，考虑赖子的数量，赖子越多，炸弹越强
        if (wildCardCountCurrent > wildCardCountPrevious) {
            return 1;  // 当前手牌有更多赖子，炸弹更强
        }
        if (wildCardCountCurrent < wildCardCountPrevious) {
            return -1; // 上一手牌有更多赖子，炸弹更强
        }

        // 如果炸弹点数和赖子数量都相同，则返回平局
        return 0;
    }

    /**
     * 获取指定频率的最大点数
     *
     * @param points 点数列表
     * @param frequency 目标频率（如 3 表示三张）
     * @return 满足频率的最大点数
     */
    private static int getMaxFrequencyCard(List<Integer> points, int frequency) {
        return points.stream()
                .collect(Collectors.groupingBy(p -> p, Collectors.counting()))
                .entrySet().stream()
                .filter(entry -> entry.getValue() >= frequency)
                .mapToInt(Map.Entry::getKey)
                .max()
                .orElse(-1); // 如果没有满足条件的点数，返回 -1
    }

    /**
     * 获取牌点
     * @param card 牌的值
     * @return 牌点
     */
    public static int getCardPoint(int card) {
        if (card == 0xEF) return 16; // 小王
        if (card == 0xFF) return 17; // 大王
        return card & 0xF;
    }

    /**
     * 获取牌点，考虑级牌影响
     * @param card 牌的值
     * @param curLevel 当前级数
     * @return 牌点
     */
    public static int getCardPoint(int card, int curLevel) {
        if (card == 0xEF) return 16; // 小王
        if (card == 0xFF) return 17; // 大王

        int point = card & 0xF; // 其他牌点
        if (point == curLevel) {
            return 15; // 级牌
        }
        return point;
    }

    /**
     * 获取单张牌的点数
     *
     * @param card 当前牌的值
     * @param wildCard 赖子牌的值
     * @param curLevel 当前级数
     * @return 返回牌的点数
     */
    public static int getCardPointWildCard(int card, int wildCard, int curLevel) {
        if (card == wildCard) {
            // 如果是赖子牌，默认返回一个通用标识值（如 -1），后续处理时动态替换
            return -1;
        }
        // 计算牌的点数
        return getCardPoint(card, curLevel);
    }


    /**
     * 获取牌型的权重值
     * @param cardType 牌型
     * @return 权重值
     */
    public static int getCardTypeWeight(CardType cardType) {
        switch (cardType) {
            case FOUR_KINGS: return 7; // 4个王 > ...
            case EIGHT_BOMB: return 6; // 8炸弹 > ...
            case SEVEN_BOMB: return 5; // 7炸弹 > ...
            case SIX_BOMB: return 4;   // 6炸弹 > ...
            case ROYAL_FLUSH: return 3;// 同花顺 > ...
            case FIVE_BOMB: return 2;  // 5炸弹 > ...
            case FOUR_BOMB: return 1;  // 4炸弹 > ...
            default: return 0; // 其他牌型
        }
    }

    /**
     * 判断牌型
     * @param disCardList 玩家出的牌（int数组）
     * @param wildCard 赖子牌
     * @return 牌型枚举值
     */
    public static CardType determineCardType(int[] disCardList, int wildCard) {
        if (disCardList == null || disCardList.length == 0) {
            return CardType.INVALID;
        }

        // 提取牌点，处理赖子（wildCard）
        List<Integer> points = new ArrayList<>();
        int wildCount = 0; // 记录赖子数量
        for (int card : disCardList) {
            if (card == wildCard) {
                wildCount++;
            } else {
                points.add(GDUtils.getCardPoint(card));
            }
        }

        // 统计牌点频率
        Map<Integer, Long> pointFrequency = points.stream()
                .collect(Collectors.groupingBy(p -> p, Collectors.counting()));

        // 排序后的牌点集合
        List<Integer> sortedPoints = points.stream()
                .distinct()
                .sorted()
                .collect(Collectors.toList());

        // 判断不同牌型
        int cardCount = disCardList.length;
        int uniqueCount = pointFrequency.size();

        // 1. 四王炸（大小王各两张）
        if (cardCount == 4 && pointFrequency.size() == 2 && points.contains(16) && points.contains(17)
                && pointFrequency.getOrDefault(16, 0L) + wildCount >= 2
                && pointFrequency.getOrDefault(17, 0L) + wildCount >= 2) {
            return CardType.FOUR_KINGS;
        }

        // 2. 特定炸弹类型（四炸弹到八炸弹）
        for (int i = 4; i <= 8; i++) {
            if (cardCount == i && uniqueCount == 1) {
                long count = pointFrequency.values().iterator().next();
                if (count + wildCount == i) {
                    switch (i) {
                        case 4: return CardType.FOUR_BOMB;
                        case 5: return CardType.FIVE_BOMB;
                        case 6: return CardType.SIX_BOMB;
                        case 7: return CardType.SEVEN_BOMB;
                        case 8: return CardType.EIGHT_BOMB;
                    }
                }
            }
        }

        // 3. 通用炸弹（四张及以上相同牌点）
        if (uniqueCount == 1 && pointFrequency.values().iterator().next() + wildCount >= 4) {
            return CardType.BOMB;
        }

        // 4. 同花顺
        if (cardCount == 5 && isRoyalFlush(disCardList, wildCard)) {
            return CardType.ROYAL_FLUSH;
        }

        // 5. 顺子（赖子可补齐缺失点数）
        if (cardCount >= 5 && uniqueCount + wildCount >= 5 && isStraightWithWild(sortedPoints, wildCount)) {
            return CardType.STRAIGHT;
        }

        // 6. 三顺（赖子可补齐缺失三张）
        if (cardCount >= 6 && cardCount % 3 == 0 && uniqueCount + wildCount >= cardCount / 3
                && isTripleSequenceWithWild(pointFrequency, wildCount)) {
            return CardType.TRIPLE_SEQUENCE;
        }

        // 7. 连对（赖子可补齐缺失对数）
        if (cardCount >= 6 && cardCount % 2 == 0 && uniqueCount + wildCount >= cardCount / 2
                && isPairSequenceWithWild(pointFrequency, wildCount)) {
            return CardType.PAIR_SEQUENCE;
        }

        // 8. 三带二
        if (cardCount == 5 && uniqueCount <= 3 && isTripleWithTwo(pointFrequency, wildCount)) {
            return CardType.TRIPLE_WITH_TWO;
        }

        // 9. 三不带
        if (cardCount == 3 && uniqueCount == 1) {
            return CardType.TRIPLE;
        }

        // 10. 对子
        if (cardCount == 2 && (uniqueCount == 1 || wildCount == 1 || (uniqueCount == 0 && wildCount == 2))) { // wildCount == 1 to form a pair
            return CardType.PAIR;
        }

        // 11. 单张
        if (cardCount == 1) {
            return CardType.SINGLE;
        }

        return CardType.INVALID; // 非法牌型
    }

    // 判断是否为三带二牌型
    private static boolean isTripleWithTwo(Map<Integer, Long> pointFrequency, int wildCount) {
        // 找出可以形成三张牌的牌点
        Optional<Map.Entry<Integer, Long>> tripleEntry = pointFrequency.entrySet().stream()
                .filter(entry -> entry.getValue() + wildCount >= 3)
                .findFirst();

        if (tripleEntry.isPresent()) {
            int tripleCount = (int) (long) tripleEntry.get().getValue(); // 当前牌点已有数量
            int neededWildsForTriple = 3 - tripleCount; // 补齐三张所需赖子数量

            if (wildCount >= neededWildsForTriple) {
                // 剩余赖子数量
                int remainingWilds = wildCount - neededWildsForTriple;

                // 统计剩余牌点的总数
                int remainingCards = pointFrequency.values().stream().mapToInt(Long::intValue).sum()
                        + remainingWilds - 3; // 扣除已经组成三张的牌

                // 判断剩余牌是否可以组成两张
                return remainingCards >= 2;
            }
        }

        return false; // 无法组成三带二
    }

    // 判断是否为顺子（赖子补齐）
    private static boolean isStraightWithWild(List<Integer> points, int wildCount) {
        // 特例1：A-2-3-4-5
        if (points.size() == 5 && points.contains(14) && points.contains(2)
                && points.contains(3) && points.contains(4) && points.contains(5)) {
            return true;
        }
        // 特例2：10-J-Q-K-A
        if (points.size() == 5 && points.contains(10) && points.contains(11)
                && points.contains(12) && points.contains(13) && points.contains(14)) {
            return true;
        }
        int gaps = 0;
        // 计算间隙
        for (int i = 0; i < points.size() - 1; i++) {
            // 如果当前点为 A（14），且下一个点为 2，视为连续
            if (points.get(i) == 14 && points.get(i + 1) == 2) {
                continue;
            }
            gaps += points.get(i + 1) - points.get(i) - 1;
        }
        // 如果包含 A，尝试作为最小点数补齐
        if (points.contains(14) && !points.contains(2)) {
            List<Integer> adjustedPoints = new ArrayList<>(points);
            adjustedPoints.remove((Integer) 14);
            adjustedPoints.add(1); // 将 A 视为点数 1
            Collections.sort(adjustedPoints);
            int adjustedGaps = 0;
            for (int i = 0; i < adjustedPoints.size() - 1; i++) {
                adjustedGaps += adjustedPoints.get(i + 1) - adjustedPoints.get(i) - 1;
            }
            if (adjustedGaps <= wildCount) {
                return true; // 作为最小点数时有效
            }
        }
        // 检查间隙是否能被赖子补齐
        return gaps <= wildCount;
    }

    // 判断是否为连对（赖子补齐）
    private static boolean isPairSequenceWithWild(Map<Integer, Long> pointFrequency, int wildCount) {
        List<Integer> sortedKeys = pointFrequency.keySet().stream().sorted().collect(Collectors.toList());
        // 特定牌型判断：A-A-2-2-3-3 是最小的三连对
        if (sortedKeys.contains(14) && sortedKeys.contains(2) && sortedKeys.contains(3)) {
            return true; // A-A-2-2-3-3 是最小的三连对
        }
        // 特定牌型判断：Q-Q-K-K-A-A 是最大的三连对
        if (sortedKeys.contains(12) && sortedKeys.contains(13) && sortedKeys.contains(14)) {
            return true; // Q-Q-K-K-A-A 是最大的三连对
        }
        int gaps = 0;
        for (int i = 0; i < sortedKeys.size() - 1; i++) {
            if (sortedKeys.get(i + 1) - sortedKeys.get(i) != 1) {
                return false; // 非连续
            }
            gaps += (int) (2 - pointFrequency.get(sortedKeys.get(i))); // 计算每个点数对缺失的牌
        }
        gaps += (int) (2 - pointFrequency.get(sortedKeys.get(sortedKeys.size() - 1)));
        return gaps <= wildCount;
    }

    // 判断是否为三顺（赖子补齐）
    private static boolean isTripleSequenceWithWild(Map<Integer, Long> pointFrequency, int wildCount) {
        List<Integer> triples = pointFrequency.keySet().stream()
                .filter(key -> pointFrequency.get(key) >= 3)
                .sorted()
                .collect(Collectors.toList());

        // 如果没有符合条件的三张连续牌，直接返回 false
        if (triples.isEmpty()) {
            return false;
        }
        // 排除特例：A-A-A-2-2-2 是有效的最小三顺
        if (triples.contains(14) && triples.contains(2)) { // A 和 2 的点数
            return true; // A-A-A-2-2-2 不是有效的三顺
        }
        // 特例：K-K-K-A-A-A 是有效的最大三顺
        if (triples.contains(13) && triples.contains(14)) { // K 和 A 的点数
            return true; // K-K-K-A-A-A 是最大的三顺
        }

        int gaps = 0;
        for (int i = 0; i < triples.size() - 1; i++) {
            // 允许跨界的情况（K-A、A-2）
            if (triples.get(i) == 14 && triples.get(i + 1) == 2) {
                continue;
            }
            if (triples.get(i + 1) - triples.get(i) != 1) {
                return false;
            }
            gaps += (int) (3 - pointFrequency.get(triples.get(i)));
        }
        // 最后一个元素也需要处理
        gaps += (int) (3 - pointFrequency.get(triples.get(triples.size() - 1)));
        return gaps <= wildCount;
    }

    // 判断是否为同花顺（可选规则）
    private static boolean isRoyalFlush(int[] cards, int wildCount) {
        return Arrays.stream(cards).map(card -> card & 0xF0).distinct().count() == 1
                && isStraightWithWild(Arrays.stream(cards).map(GDUtils::getCardPoint).sorted().boxed().collect(Collectors.toList()), wildCount);
    }

    // 测试代码
    public static void main(String[] args) {
        int[] cards1 = {0x08, 0x04, 0x05, 0x06, 0x07}; // 顺子
        int[] cards2 = {0x33, 0x13, 0x23, 0x03};       // 四炸弹
        int[] cards8 = {0xEF, 0xEF, 0xFF, 0xFF};             // 四王炸
        int[] cards9 = {0x03, 0x13, 0x23, 0x33};             // 四炸弹
        int[] cards10 = {0x04, 0x14, 0x24, 0x34, 0x44};      // 五炸弹
        int[] cards11 = {0x05, 0x15, 0x25, 0x35, 0x45, 0x55}; // 六炸弹
        int[] cards12 = {0x06, 0x16, 0x26, 0x36, 0x46, 0x56, 0x66}; // 七炸弹
        int[] cards13 = {0x07, 0x17, 0x27, 0x37, 0x47, 0x57, 0x67, 0x77}; // 八炸弹
        int[] cards14 = {0x0e, 0x12, 0x23, 0x04, 0x05}; // 顺子
        int[] cards15 = {0x02, 0x02, 0x03, 0x03, 0x0e, 0x0e}; // A-A-2-2-3-3

        System.out.println(determineCardType(cards1, 0)); // STRAIGHT
        System.out.println(determineCardType(cards2, 0)); // BOMB
        System.out.println(determineCardType(cards8, 0)); // FOUR_KINGS
        System.out.println(determineCardType(cards9, 0)); // FOUR_BOMB
        System.out.println(determineCardType(cards10, 0)); // FIVE_BOMB
        System.out.println(determineCardType(cards11, 0)); // SIX_BOMB
        System.out.println(determineCardType(cards12, 0)); // SEVEN_BOMB
        System.out.println(determineCardType(cards13, 0)); // EIGHT_BOMB
        System.out.println(determineCardType(cards14, 0)); // STRAIGHT
        System.out.println(determineCardType(cards15, 0)); // PAIR_SEQUENCE

        int[] cards20 = {0x02, 0x02, 0x02, 0x0e, 0x0e, 0x0e}; // A-A-A-2-2-2
        int[] cards21 = {0x03, 0x03, 0x03, 0x02, 0x02, 0x02}; // 3-3-3-4-4-4

        System.out.println(compareCardStrength(cards20, cards21, 3, CardType.TRIPLE_SEQUENCE, 0));
    }
}